/*    proto.h
 *
 *    Copyright (C) 1991, 1996, by Larry Wall
 *
 *    You may distribute under the terms of either the GNU General Public
 *    License or the Artistic License, as specified in the README file.
 *
 */
